/* IMPORTANT
 * This snapshot file is auto-generated, but designed for humans.
 * It should be checked into source control and tracked carefully.
 * Re-generate by setting TAP_SNAPSHOT=1 and running tests.
 * Make sure to inspect the output below.  Do not ignore changes!
 */
'use strict'
exports[`test/basic.ts TAP basic operation > must match snapshot 1`] = `
Generator [
  Array [
    4,
    4,
  ],
  Array [
    3,
    3,
  ],
  Array [
    2,
    2,
  ],
  Array [
    1,
    1,
  ],
  Array [
    0,
    0,
  ],
]
`

exports[`test/basic.ts TAP basic operation > must match snapshot 2`] = `
Generator [
  Array [
    9,
    9,
  ],
  Array [
    8,
    8,
  ],
  Array [
    7,
    7,
  ],
  Array [
    6,
    6,
  ],
  Array [
    5,
    5,
  ],
  Array [
    4,
    4,
  ],
  Array [
    3,
    3,
  ],
  Array [
    2,
    2,
  ],
  Array [
    1,
    1,
  ],
  Array [
    0,
    0,
  ],
]
`

exports[`test/basic.ts TAP basic operation > must match snapshot 3`] = `
Generator [
  Array [
    4,
    4,
  ],
  Array [
    3,
    3,
  ],
  Array [
    2,
    2,
  ],
  Array [
    1,
    1,
  ],
  Array [
    0,
    0,
  ],
  Array [
    9,
    9,
  ],
  Array [
    8,
    8,
  ],
  Array [
    7,
    7,
  ],
  Array [
    6,
    6,
  ],
  Array [
    5,
    5,
  ],
]
`

exports[`test/basic.ts TAP basic operation > must match snapshot 4`] = `
Generator [
  Array [
    14,
    14,
  ],
  Array [
    13,
    13,
  ],
  Array [
    12,
    12,
  ],
  Array [
    11,
    11,
  ],
  Array [
    10,
    10,
  ],
  Array [
    9,
    9,
  ],
  Array [
    8,
    8,
  ],
  Array [
    7,
    7,
  ],
  Array [
    6,
    6,
  ],
  Array [
    5,
    5,
  ],
]
`

exports[`test/basic.ts TAP basic operation > must match snapshot 5`] = `
Generator [
  Array [
    19,
    19,
  ],
  Array [
    18,
    18,
  ],
  Array [
    17,
    17,
  ],
  Array [
    16,
    16,
  ],
  Array [
    15,
    15,
  ],
  Array [
    14,
    14,
  ],
  Array [
    13,
    13,
  ],
  Array [
    12,
    12,
  ],
  Array [
    11,
    11,
  ],
  Array [
    10,
    10,
  ],
]
`

exports[`test/basic.ts TAP basic operation > must match snapshot 6`] = `
Generator [
  Array [
    19,
    19,
  ],
  Array [
    18,
    18,
  ],
  Array [
    17,
    17,
  ],
  Array [
    16,
    16,
  ],
  Array [
    15,
    15,
  ],
  Array [
    14,
    14,
  ],
  Array [
    13,
    13,
  ],
  Array [
    12,
    12,
  ],
  Array [
    11,
    11,
  ],
  Array [
    10,
    10,
  ],
]
`

exports[`test/basic.ts TAP basic operation > status tracking 1`] = `
Array [
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "get": "hit",
  },
  Object {
    "get": "hit",
  },
  Object {
    "get": "hit",
  },
  Object {
    "get": "hit",
  },
  Object {
    "get": "hit",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "update",
  },
  Object {
    "set": "update",
  },
  Object {
    "set": "update",
  },
  Object {
    "set": "update",
  },
  Object {
    "set": "update",
  },
  Object {
    "get": "hit",
  },
  Object {
    "get": "hit",
  },
  Object {
    "get": "hit",
  },
  Object {
    "get": "hit",
  },
  Object {
    "get": "hit",
  },
  Object {
    "get": "hit",
  },
  Object {
    "get": "hit",
  },
  Object {
    "get": "hit",
  },
  Object {
    "get": "hit",
  },
  Object {
    "get": "hit",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "get": "miss",
  },
  Object {
    "get": "miss",
  },
  Object {
    "get": "miss",
  },
  Object {
    "get": "miss",
  },
  Object {
    "get": "miss",
  },
  Object {
    "get": "miss",
  },
  Object {
    "get": "miss",
  },
  Object {
    "get": "miss",
  },
  Object {
    "get": "miss",
  },
  Object {
    "get": "miss",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "has": "hit",
  },
  Object {
    "set": "add",
  },
  Object {
    "has": "hit",
  },
  Object {
    "get": "hit",
  },
  Object {
    "has": "miss",
  },
]
`

exports[`test/basic.ts TAP re-use key before initial fill completed > must match snapshot 1`] = `
Array [
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "set": "add",
  },
  Object {
    "oldValue": 1,
    "set": "replace",
  },
  Object {
    "set": "add",
  },
]
`
